/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   Recalculador de maildirsize                                         
   Programado por Lucas Priori                                         
   Versao: 1.0                                                         
                                                                       
   Compilar:                                                           
   gcc rmaildirsize.c -o rmaildirsize                                  
                                                                       
  ******************************************************************   
  rmaildirsize  um programa que analisa uma caixa postal Maildir      
  e arruma seu respectivo arquivo maildirsize                          
  Copyright (C) 2007  Lucas Priori                                     
                                                                       
  This program is free software: you can redistribute it and/or modify 
  it under the terms of the GNU General Public License as published by 
  the Free Software Foundation, either version 3 of the License, or    
  (at your option) any later version.                                  
                                                                       
  This program is distributed in the hope that it will be useful,      
  but WITHOUT ANY WARRANTY; without even the implied warranty of       
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
  GNU General Public License for more details.                         
                                                                       
  You should have received a copy of the GNU General Public License    
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<dirent.h>
#include<fcntl.h>
#include<unistd.h>
#include<sys/stat.h>

#define VERSAO "1.0"
#define TAM_buffer 2048





// =====================================================
// Funcao de contagem dos emails                        
// Retorna em nmsgs o numero de mensagens e em tmsgs o  
// tamanho total de todas as mensagens da pasta         
// =====================================================
void cont_emails(char *caminho,long long int *nmsgs, long long int *tmsgs){
 DIR *dir;
 struct dirent *entr;
 struct stat dados_entr;
 long long int num_temp=0;
 long long int tam_temp=0;
 
 dir=opendir(caminho);
 if(!dir){
  *nmsgs=0;
  *tmsgs=0;
  return ;
 }
 printf("     %s",caminho);
 readdir(dir);        // Retirando o '.' e '..' para nao precisar de if 
 readdir(dir);        //  dentro do while                               
 chdir(caminho);      // Mudando o diretorio corrente para nao precisar 
                      // especificar caminho usando o sprintf           
 while((entr=readdir(dir))!= NULL){
  stat(entr->d_name,&dados_entr);
  num_temp++;
  tam_temp+=dados_entr.st_size;
 }
 *nmsgs=num_temp;
 *tmsgs=tam_temp;
 closedir(dir);
 printf("  Msgs: %lld   Tam: %lld\n",num_temp,tam_temp);
}




// =====================================================
// Funcao principal                                     
// =====================================================
int main(int argc, char *argv[]){
 DIR *dir;
 struct dirent *entr;
 struct stat dados_entr;
 char buffer[TAM_buffer+1];
 char linha[TAM_buffer+1];
 char *ptr_aux1=0;
 int arquivo=0;
 unsigned int cont1=1;
 unsigned int cont2=0;
 long long int num_msgs=0;
 long long int tam_msgs=0;
 long long int num_total=0;
 long long int tam_total=0;
 
 if(argc==1){
  printf("Rmaildirsize v %s\n",VERSAO);
  printf("Voce precisa especificar os caminhos dos diretorios das caixas postais\n");
  printf("Exemplos:\n");
  printf(" ./rmaildirsize /home/mail/meudominio/contato/Maildir /home/mail/meudominio/adm/Maildir\n");
  printf(" ./rmaildirsize /home/mail/meudominio/*/Maildir\n");
  printf(" ./rmaildirsize /home/mail/*/*/Maildir\n\n");
  return 0;
 }
 for(cont1=1;cont1<argc;cont1++){
  printf("--> [%s]      \n",argv[cont1]);
  num_total=0;
  tam_total=0;
  dir=opendir(argv[cont1]);
  if(dir){
   while((entr = readdir(dir)) != NULL){
    // Contando emails de pastas particulares 
    if(entr->d_name[0]!='.' || (entr->d_name[1]!='.' && entr->d_name[1])){
     ptr_aux1=buffer+snprintf(buffer,TAM_buffer,"%s/%s",argv[cont1],entr->d_name);
     stat(buffer,&dados_entr);
     if(dados_entr.st_mode & S_IFDIR){
      if(entr->d_name[0]=='.'){
       strcpy(ptr_aux1,"/new");
       cont_emails(buffer,&num_msgs,&tam_msgs);
       num_total+=num_msgs;
       tam_total+=tam_msgs;
       strcpy(ptr_aux1,"/cur");
       cont_emails(buffer,&num_msgs,&tam_msgs);
       num_total+=num_msgs;
       tam_total+=tam_msgs;
       strcpy(ptr_aux1,"/tmp");
       cont_emails(buffer,&num_msgs,&tam_msgs);
       num_total+=num_msgs;
       tam_total+=tam_msgs;
      }
     }
    }
   }
   // Contando emails da INBOX 
   ptr_aux1=buffer+snprintf(buffer,TAM_buffer,"%s",argv[cont1]);
   strcpy(ptr_aux1,"/new");
   cont_emails(buffer,&num_msgs,&tam_msgs);
   num_total+=num_msgs;
   tam_total+=tam_msgs;
   strcpy(ptr_aux1,"/cur");
   cont_emails(buffer,&num_msgs,&tam_msgs);
   num_total+=num_msgs;
   tam_total+=tam_msgs;
   strcpy(ptr_aux1,"/tmp");
   cont_emails(buffer,&num_msgs,&tam_msgs);
   num_total+=num_msgs;
   tam_total+=tam_msgs;
   closedir(dir);
   printf("     TOTAL: MSGS %lld  TAM %lld (%lld MB)",num_total,tam_total,tam_total>>20);
   snprintf(buffer,TAM_buffer,"%s/maildirsize",argv[cont1]);
   arquivo=open(buffer,O_RDWR | O_CREAT,0777);
   if(arquivo){
    if(!lockf(arquivo,F_TLOCK,0)){
     cont2=read(arquivo,linha,TAM_buffer/2);
     if(!strchr(linha,'S') && !strchr(linha,'C')){
      ptr_aux1=linha;
      *ptr_aux1=10;
     }else if(!(ptr_aux1=strchr(linha,10))){
      ptr_aux1=linha+cont2;
      *ptr_aux1=10;
     }
     ptr_aux1++;
     cont2=ptr_aux1-linha;
     cont2+=sprintf(ptr_aux1,"%12lld %12lld\n",tam_total,num_total);
     lseek(arquivo,0,0);
     write(arquivo,linha,cont2);
     close(arquivo);
     truncate(buffer,cont2);
     printf(" -->  maildirsize atualizado\n");
    }else printf(" -->  maildirsize em uso...\n");
   }else printf("    Nao foi possivel abrir/criar o maildirsize\n");
  }else printf("INEXISTENTE\n");
 }
 return 0;
}






